#!/usr/bin/env python

# 321=134.197.40.0/22
# 231=172.20.195.0/22
import multiprocessing as mp
import pprint
import nmap
import json
import zmq
import time
import os

from netaddr import IPNetwork

NUM_PROCS = 32

def portScan(iqueue, oqueue):
  while True:
    msg = iqueue.get()         # Read from the queue and do nothing
    if msg == 'exit':
      break

    nm = nmap.PortScanner() # instantiate nmap.PortScanner object
    nm.scan(msg, arguments='-sn')

    hosts = nm.all_hosts()

    thosts = []
    for h in hosts:
        thosts.append(nm[h])

    print("Hosts: %s" % thosts)
    ctx = zmq.Context()
    s = ctx.socket(zmq.PUSH)
    s.connect(b"tcp://172.20.195.1:4900")

    topic = "nmap"
    msg_body = bytes(json.dumps(thosts), 'UTF-8')

    time.sleep(1)
    print('   Topic: %s' % (topic))
    s.send_multipart([bytes(topic, 'UTF-8'), msg_body])

    print("Waiting for message queues to flush...")
    time.sleep(1)
    print("Done.")

def main():
  pp = pprint.PrettyPrinter(indent=4)
  iqueue = mp.Queue() #input queue
  oqueue = mp.Queue() #output queue
  procs = []


  for i in range(0,NUM_PROCS):
    p = mp.Process(target=portScan, args=((iqueue),oqueue))
    procs.append(p)
    p.start()
  ips = IPNetwork('172.20.195.0/22')
  [iqueue.put("%s" % ip) for ip in ips]
  [iqueue.put("exit") for i in range(0, NUM_PROCS)]

  for p in procs:
    p.join()


  os.system('stty sane')



if __name__ == "__main__":
  main()
